from setuptools import find_packages, setup

setup(
    name='matuamod_serializer',
    packages=['matuamod_serializer'],
    version='0.2.2',
    description='My json, toml, yaml serializer/parser library',
    author='matuamod',
    author_email='matua.models2003@gmail.com',
    url='https://github.com/matuamod/ISP-2022-053504/',
    license='MIT',
)
